package leoTest;

import com.dukascopy.api.JFException;
import com.dukascopy.api.*;
import java.awt.Color;
import com.dukascopy.api.IChart;
import com.dukascopy.api.drawings.IPolyLineChartObject;

public class LeoTestStrategy implements IStrategy {

    private IConsole console;
    private IChart chart;
       
    @Configurable("instrument")
    public Instrument instrument = Instrument.EURUSD;
    @Configurable("period")
    public Period period = Period.TICK;

    ChartTruncatingPolyline testPolyline = new ChartTruncatingPolyline();    
    
    public void onStart(IContext context) throws JFException {
        this.console = context.getConsole();
        
        this.chart = context.getChart(instrument);       
        if(chart == null) {
            context.getConsole().getErr().println("No chart opened for "+instrument);
            context.stop(); /* stop the strategy */
        } else {
            console.getOut().println("Chart opened for "+instrument+" - start processing");
        } 
        testPolyline.initPolyline(this.chart, 1, LineStyle.SOLID, Color.GREEN);        
    }

    public void onAccount(IAccount account) throws JFException {
    }

    public void onMessage(IMessage message) throws JFException {
    }

    public void onStop() throws JFException {
    }

    public void onTick(Instrument instrument, ITick tick) throws JFException {
        if (instrument != this.instrument){
            return;
        }
        /* connect price of incoming ticks
         */      
        testPolyline.nextPoint(tick.getTime(), (tick.getBid()+tick.getAsk())/2);
    }
    
    public void onBar(Instrument instrument, Period period, IBar askBar, IBar bidBar) throws JFException {            
    }
    
    public class ChartTruncatingPolyline {
        private IPolyLineChartObject valuesLine; 
        
        void initPolyline(IChart chart, float lineWidth, int lineStyle, Color color) throws JFException {
                valuesLine = chart.getChartObjectFactory().createPolyLine();     
                valuesLine.setLineWidth(lineWidth);
                valuesLine.setLineStyle(lineStyle);
                valuesLine.setColor(color);
                chart.add(valuesLine);            
        }
        void nextPoint(long time, double value) throws JFException {
            valuesLine.addNewPoint(time, value);          
            while (valuesLine.getPointsCount() >= IPolyLineChartObject.MAX_POINTS_COUNT){
                valuesLine.removePoint(0);
            }
        }
    }   
}